@@ -34,7 +34,6 @@ dataset$key_upload_status <- "toy"
 # toy data is uploaded when user clicks on different toy dataset or in case that
 # user clicks on "Unload data" button
 observeEvent(c(input$data_toydata, data_csvdata_current_status$unloaded == 1), {
-
   toydata <- input$data_toydata
   toydata_name <- strsplit(toydata, split = "_")[[1]][1]
   toydata_package <- strsplit(toydata, split = "_")[[1]][2]
@@ -42,7 +41,48 @@ observeEvent(c(input$data_toydata, data_csvdata_current_status$unloaded == 1), {
 
   dataset$data_status <- "OK"
 
-  if (toydata_name == "LearningToLearn" & toydata_subset == "6") {
+  if (toydata_name == "CLoSEread6") {
+    # ** CLoSEread6 ####
+    toydata_data_type <- "binary"
+
+    do.call(data, args = list(paste0(toydata_name), package = toydata_package))
+    toydata_binary <- get(paste0(toydata_name))[1:19]
+
+    toydata_ordinal <- toydata_binary
+    toydata_continuous <- toydata_ordinal
+    toydata_nominal <- toydata_ordinal
+
+    toydata_group <- get(paste0(toydata_name))[, "gender"]
+    toydata_criterion <- "missing"
+    toydata_DIFmatching <- "missing"
+
+    toydata_minimal <- NULL
+    toydata_maximal <- NULL
+
+    toydata_key <- rep(1, ncol(toydata_binary))
+  } else if (toydata_name == "CZmaturaS") {
+
+    toydata_data_type <- "ordinal"
+
+    # get the dataset directly from the package's namespace without changing the .GlobalEnv
+    toydata_raw <- get(toydata_name, envir = asNamespace(toydata_package))
+
+    # use already scored responses
+    toydata_ordinal <- toydata_raw %>% select(matches("b\\d+"))
+    toydata_continuous <- toydata_ordinal
+    toydata_nominal <- toydata_ordinal
+
+    toydata_group <- toydata_raw[, "SchTypeGY"]
+    toydata_criterion <- toydata_raw[, "SchTypeGY"]
+    # rounded T-scores because the plots demand discrete values
+    toydata_DIFmatching <- round((scale(toydata_raw[, "IRTscore"]) * 10) + 50)
+
+    toydata_minimal <- sapply(toydata_ordinal, min, na.rm = TRUE)
+    toydata_key <- toydata_maximal <- sapply(toydata_ordinal, max, na.rm = TRUE)
+
+    toydata_binary <- mirt::key2binary(toydata_ordinal, toydata_key)
+
+  } else if (toydata_name == "LearningToLearn" & toydata_subset == "6") {
     # ** Learning to learn, grade 6 ####
     toydata_data_type <- "binary"
 
@@ -61,7 +101,6 @@ observeEvent(c(input$data_toydata, data_csvdata_current_status$unloaded == 1), {
     toydata_maximal <- NULL
 
     toydata_key <- rep(1, ncol(toydata_binary))
-
   } else if (toydata_name == "LearningToLearn" & toydata_subset == "9") {
     # ** Learning to learn, grade 9 ####
     toydata_data_type <- "binary"
@@ -82,7 +121,6 @@ observeEvent(c(input$data_toydata, data_csvdata_current_status$unloaded == 1), {
     toydata_maximal <- NULL
 
     toydata_key <- rep(1, ncol(toydata_binary))
-
   } else if (toydata_name == "dataMedicalgraded") {
     # ** Medical graded ####
     toydata_data_type <- "ordinal"
@@ -105,7 +143,6 @@ observeEvent(c(input$data_toydata, data_csvdata_current_status$unloaded == 1), {
     # key2binary is much more faster than the old approach, but it is
     # only usable when maximum score is considered as the key
     toydata_binary <- mirt::key2binary(toydata_ordinal, toydata_key)
-
   } else if (toydata_name == "Science") {
     # ** Science ####
     toydata_data_type <- "ordinal"
@@ -127,8 +164,30 @@ observeEvent(c(input$data_toydata, data_csvdata_current_status$unloaded == 1), {
     # key2binary is much more faster than the old approach, but it is
     # only usable when maximum score is considered as the key
     toydata_binary <- mirt::key2binary(toydata_ordinal, toydata_key)
+  } else if (toydata_name == "Anxiety") {
+    # ** Anxiety ####
+    toydata_data_type <- "ordinal"
 
-  } else if (toydata_name == "AIBS") {
+    do.call(data, args = list(paste0(toydata_name), package = toydata_package))
+
+    toydata_ordinal <- get(paste0(toydata_name))
+
+    toydata_group <- toydata_ordinal[, "gender"]
+    toydata_criterion <- "missing"
+    toydata_DIFmatching <- "missing"
+
+    toydata_ordinal <- toydata_ordinal[, paste0("R", 1:29)]
+    toydata_continuous <- toydata_ordinal
+    toydata_nominal <- toydata_ordinal
+
+    toydata_minimal <- sapply(toydata_ordinal, min, na.rm = TRUE)
+    toydata_maximal <- sapply(toydata_ordinal, max, na.rm = TRUE)
+
+    toydata_key <- rep(2, 29) # never vs. at least rarely
+    # key2binary is much more faster than the old approach, but it is
+    # only usable when maximum score is considered as the key
+    toydata_binary <- mirt::key2binary(toydata_ordinal, toydata_key)
+  } else if (toydata_name == "AIBS") { # not used
     # ** AIBS ####
     toydata_data_type <- "continuous"
 
@@ -147,7 +206,6 @@ observeEvent(c(input$data_toydata, data_csvdata_current_status$unloaded == 1), {
     toydata_maximal <- "missing"
 
     toydata_key <- "missing"
-
   } else {
     # ** Nominal datasets - GMAT, HCI, MSATB, Medical 100 ####
     toydata_data_type <- "nominal"
@@ -175,10 +233,20 @@ observeEvent(c(input$data_toydata, data_csvdata_current_status$unloaded == 1), {
     toydata_binary <- toydata_ordinal
   }
 
+  # TODO we don't perform any expensive operations on data frames,
+  # so data.table is a bit superfluous and can be dropped as a dependency
+  # instead, we should keep an eye on the classes (numeric takes more memory
+  # than integers etc.) and possibly replace read.csv with something faster ({vroom})
+  # especially for factors, data.table makes smaller objects than base R,
+  # but tibbles are at the same level and underliyng {vctrs} makes everything lightning-fast
+
   # ** Saving into reactiveValues ####
   dataset$nominal <- as.data.table(toydata_nominal)
   dataset$ordinal <- as.data.table(toydata_ordinal)
   dataset$binary <- as.data.table(toydata_binary)
+
+  # continuous is not supported and is a relic of AIBS introduction, which
+  # is now "solved" using SIA module
   dataset$continuous <- as.data.table(toydata_continuous)
 
   dataset$name <- toydata_name
@@ -198,7 +266,6 @@ observeEvent(c(input$data_toydata, data_csvdata_current_status$unloaded == 1), {
 
 # toy data is uploaded when user clicks on "Upload data" button
 observeEvent(input$data_upload, {
-
   csvdata_data <- NULL
   csvdata_key <- NULL
   csvdata_group <- NULL
@@ -233,6 +300,11 @@ observeEvent(input$data_upload, {
     dataset$data_status <- "OK"
     csvdata_data_name <- gsub(".csv", "", input$data_csvdata_main[[1]])
 
+    # ensure data are truly binary (integers are faster than numeric)
+    if (input$data_csvdata_data_type == "binary") {
+      csvdata_data <- modify(csvdata_data, as.integer)
+    }
+
     # ** Loading minimal/maximual values for ordinal data ####
     if (input$data_csvdata_data_type == "ordinal") {
       # changing factors to numeric
@@ -504,9 +576,10 @@ nominal <- reactive({
 continuous <- reactive({
   data <- dataset$continuous
 
-  if (!input$data_csvdata_keep_missing) {
-    data[is.na(data)] <- 0
+  if (input$data_csvdata_replace_missing) {
+    data[is.na(data)] <- 0L
   }
+
   data
 })
 
@@ -522,8 +595,9 @@ ordinal <- reactive({
     errorClass = "validation-error"
   )
 
-  if (!input$data_csvdata_keep_missing) {
-    data[is.na(data)] <- 0
+
+  if (input$data_csvdata_replace_missing) {
+    data[is.na(data)] <- 0L
   }
   data
 })
@@ -540,8 +614,8 @@ binary <- reactive({
     errorClass = "validation-error"
   )
 
-  if (!input$data_csvdata_keep_missing) {
-    data[is.na(data)] <- 0
+  if (input$data_csvdata_replace_missing) {
+    data[is.na(data)] <- 0L
   }
   data
 })
@@ -559,11 +633,12 @@ key <- reactive({
     )
   } else {
     # incorrect dimension of key
-    validate(need(
-      ncol(nominal()) == length(dataset$key),
-      "The length of the key needs to be the same as the number of the items of the main dataset!"
-    ),
-    errorClass = "validation-error"
+    validate(
+      need(
+        ncol(nominal()) == length(dataset$key),
+        "The length of the key needs to be the same as the number of the items of the main dataset!"
+      ),
+      errorClass = "validation-error"
     )
   }
   dataset$key
@@ -573,11 +648,12 @@ key <- reactive({
 minimal <- reactive({
   if (!is.null(dataset$minimal)) {
     # incorrect dimension of minimal values
-    validate(need(
-      ncol(nominal()) == length(dataset$minimal),
-      "The length of minimal values needs to be the same as the number of items in the main dataset!"
-    ),
-    errorClass = "validation-error"
+    validate(
+      need(
+        ncol(nominal()) == length(dataset$minimal),
+        "The length of minimal values needs to be the same as the number of items in the main dataset!"
+      ),
+      errorClass = "validation-error"
     )
   }
   dataset$minimal
@@ -587,11 +663,12 @@ minimal <- reactive({
 maximal <- reactive({
   if (!is.null(dataset$minimal)) {
     # incorrect dimension of maximal values
-    validate(need(
-      ncol(nominal()) == length(dataset$maximal),
-      "The length of maximal values needs to be the same as the number of items in the main dataset!"
-    ),
-    errorClass = "validation-error"
+    validate(
+      need(
+        ncol(nominal()) == length(dataset$maximal),
+        "The length of maximal values needs to be the same as the number of items in the main dataset!"
+      ),
+      errorClass = "validation-error"
     )
   }
   dataset$maximal
@@ -599,21 +676,23 @@ maximal <- reactive({
 
 # ** Group ####
 group <- reactive({
-  if (length(dataset$group) == 1 & any(dataset$group == "missing")) {
+  if (length(dataset$group) == 1L & any(dataset$group == "missing")) {
     # missing group
-    validate(need(
-      dataset$group != "missing",
-      "The group variable is not provided in your data, the DIF and DDF analyses are not available."
-    ),
-    errorClass = "validation-warning"
+    validate(
+      need(
+        dataset$group != "missing",
+        "The group variable is not provided in your data, the DIF and DDF analyses are not available."
+      ),
+      errorClass = "validation-warning"
     )
   } else {
     # incorrect dimension of group variable
-    validate(need(
-      nrow(nominal()) == length(dataset$group),
-      "The length of the group variable needs to be the same as the number of observations in the main dataset!"
-    ),
-    errorClass = "validation-error"
+    validate(
+      need(
+        nrow(nominal()) == length(dataset$group),
+        "The length of the group variable needs to be the same as the number of observations in the main dataset!"
+      ),
+      errorClass = "validation-error"
     )
   }
   dataset$group
@@ -621,21 +700,23 @@ group <- reactive({
 
 # ** Criterion ####
 criterion <- reactive({
-  if (length(dataset$criterion) == 1 & any(dataset$criterion == "missing")) {
+  if (length(dataset$criterion) == 1L & any(dataset$criterion == "missing")) {
     # missing criterion
-    validate(need(
-      dataset$criterion != "missing",
-      "The criterion variable is not provided in your data, the criterion validity analysis is not available."
-    ),
-    errorClass = "validation-warning"
+    validate(
+      need(
+        dataset$criterion != "missing",
+        "The criterion variable is not provided in your data, the criterion validity analysis is not available."
+      ),
+      errorClass = "validation-warning"
     )
   } else {
     # incorrect dimension of criterion variable
-    validate(need(
-      nrow(nominal()) == length(dataset$criterion),
-      "The length of the criterion variable needs to be the same as the number of observations in the main dataset!"
-    ),
-    errorClass = "validation-error"
+    validate(
+      need(
+        nrow(nominal()) == length(dataset$criterion),
+        "The length of the criterion variable needs to be the same as the number of observations in the main dataset!"
+      ),
+      errorClass = "validation-error"
     )
   }
   dataset$criterion
@@ -650,19 +731,21 @@ crit_wo_val <- reactive({
 DIFmatching <- reactive({
   if (length(dataset$DIFmatching) == 1 & any(dataset$DIFmatching == "missing")) {
     # missing DIF matching variable
-    validate(need(
-      dataset$DIFmatching != "missing",
-      "The observed score variable is not provided in your data, the analyses will use the total scores."
-    ),
-    errorClass = "validation-warning"
+    validate(
+      need(
+        dataset$DIFmatching != "missing",
+        "The observed score variable is not provided in your data, the analyses will use the total scores."
+      ),
+      errorClass = "validation-warning"
     )
   } else {
     # incorrect dimension of DIF matching variable
-    validate(need(
-      nrow(nominal()) == length(dataset$DIFmatching), # changed to binary from nominal
-      "The length of the observed score variable needs to be the same as the number of observations in the main dataset!"
-    ),
-    errorClass = "validation-error"
+    validate(
+      need(
+        nrow(nominal()) == length(dataset$DIFmatching), # changed to binary from nominal
+        "The length of the observed score variable needs to be the same as the number of observations in the main dataset!"
+      ),
+      errorClass = "validation-error"
     )
   }
   dataset$DIFmatching
@@ -1071,6 +1154,22 @@ output$data_check_group_withNA_confirmation <- renderUI({
 data_description_Input <- reactive({
   data_name <- input$data_toydata
   txt <- switch(data_name,
+    Anxiety_ShinyItemAnalysis = "<code>Anxiety</code> is a real dataset originally from the <code>lordif</code> package.
+                This dataset contains responses of 766 respondents (369 males, coded as <code>'0'</code>, 397 females coded
+                as <code>'1'</code>) sampled from a general population to the PROMIS Anxiety scale on 29 Likert-scale items
+                with a common rating scale (<code>'1'</code> = Never, <code>'2'</code> = Rarely, <code>'3'</code> = Sometimes,
+                <code>'4'</code> = Often, and <code>'5'</code> = Always). For analyses where dichotomous items are necessary
+                (e.g., logistic models in Regression, IRT models, or DIF detection methods), data are binarized &ndash;
+                <code>'1'</code> means at least rarely on original scale, i.e., <code>'2'</code>-<code>'5'</code>; otherwise
+                the item is scored as <code>'0'</code>. ",
+    CLoSEread6_ShinyItemAnalysis = "<code>CLoSEread6</code> is a real dataset containing responses of 2,634 students (1,324 boys, coded
+                as <code>'0'</code>, 1,310 girls coded as <code>'1'</code>) on 19 dichotomously scored items in a test of
+                reading skills, version B, taken in the 6th grade (Hladka, Martinkova, & Magis, 2023). ",
+    CZmaturaS_ShinyItemAnalysis = "<code>CZmaturaS</code> is a real dataset containing responses of a random subset of 2,000 students in Grade 13 taking
+                a \"matura\" exam in mathematics. Students responded to a mixture of 26 dichotomous and polytomous items which we consider ordinal in the
+                app. For analyses that use binary data, the highest observed score in each item is scored as \"1\" and the rest as \"0\". In the Group and Criterion variable,
+                \"1\" denotes that the student attends an academic \"gymnasium\" school type. T-scores of factor scores estimated from the GPCM/2PL IRT model come
+                preloaded as an alternative observed score variable. ",
     GMAT_difNLR = "<code>GMAT</code> <a href='https://doi.org/10.1187/cbe.16-10-0307' target='_blank'>(Martinkova et al., 2017)</a>
                 is a generated dataset based on the parameters of a real Graduate Management Admission Test (GMAT; Kingston et al., 1985)
                 from the <code>difNLR</code> package. This dataset represents the responses of 2,000 subjects (1,000 males coded as
@@ -1288,12 +1387,15 @@ output$data_DIFmatching_summary <- renderPrint({
 # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 # * Main dataset ####
+
+data_exploration_main_input <- reactive({
+  data_table <- nominal()
+  colnames(data_table) <- item_names()
+  data_table
+})
+
 output$data_exploration_main <- DT::renderDataTable(
-  {
-    data_table <- nominal()
-    colnames(data_table) <- item_names()
-    data_table
-  },
+  data_exploration_main_input(),
   rownames = FALSE,
   style = "bootstrap",
   options = list(
@@ -1305,6 +1407,16 @@ output$data_exploration_main <- DT::renderDataTable(
   )
 )
 
+output$data_exploration_main_db <- downloadHandler(
+  filename = function() "main_dataset.csv",
+  content = function(file) {
+    write.csv(data_exploration_main_input(), file,
+      row.names = FALSE, quote = FALSE
+    )
+  }
+)
+
+
 # * Key ####
 output$data_exploration_key <- DT::renderDataTable(
   {
@@ -1323,13 +1435,24 @@ output$data_exploration_key <- DT::renderDataTable(
   )
 )
 
-# * Binary data with total score ####
+
+output$data_exploration_key_db <- downloadHandler(
+  filename = function() "key.csv",
+  content = function(file) {
+    data.frame(key = key()) %>%
+      write.csv(file = file, row.names = FALSE, quote = FALSE)
+  }
+)
+
+# * Binary data ####
+data_exploration_binary_input <- reactive({
+  data_table <- binary()
+  colnames(data_table) <- item_names()
+  data_table
+})
+
 output$data_exploration_binary <- DT::renderDataTable(
-  {
-    data_table <- binary()
-    colnames(data_table) <- item_names()
-    data_table
-  },
+  data_exploration_binary_input(),
   rownames = FALSE,
   style = "bootstrap",
   options = list(
@@ -1341,16 +1464,26 @@ output$data_exploration_binary <- DT::renderDataTable(
   )
 )
 
+output$data_exploration_binary_db <- downloadHandler(
+  filename = function() "scored_dataset.csv",
+  content = function(file) {
+    data_exploration_binary_input() %>%
+      write.csv(file = file, row.names = FALSE, quote = FALSE)
+  }
+)
+
 # all vector variables into one DT
+data_exploration_variables_input <- reactive({
+  tibble(
+    Group = ifelse(dataset$group != "missing", group(), "NA"),
+    Criterion = ifelse(dataset$criterion != "missing", criterion(), "NA"),
+    "Total score" = total_score(),
+    "Observed score vector" = ifelse(dataset$DIFmatching != "missing", DIFmatching(), "NA")
+  )
+})
+
 output$data_exploration_variables <- DT::renderDataTable(
-  {
-    tibble(
-      Group = ifelse(dataset$group != "missing", group(), "NA"),
-      Criterion = ifelse(dataset$criterion != "missing", criterion(), "NA"),
-      "Total score" = total_score(),
-      "Observed score vector" = ifelse(dataset$DIFmatching != "missing", DIFmatching(), "NA")
-    )
-  },
+  data_exploration_variables_input(),
   rownames = TRUE,
   style = "bootstrap",
   options = list(
@@ -1362,3 +1495,45 @@ output$data_exploration_variables <- DT::renderDataTable(
     dom = "tipr"
   )
 )
+
+
+
+
+output$data_exploration_group_db <- downloadHandler(
+  filename = function() "group.csv",
+  content = function(file) {
+    data_exploration_variables_input() %>%
+      select(group = Group) %>%
+      write.csv(file = file, row.names = FALSE, quote = FALSE)
+  }
+)
+
+
+output$data_exploration_criterion_db <- downloadHandler(
+  filename = function() "criterion.csv",
+  content = function(file) {
+    data_exploration_variables_input() %>%
+      select(criterion = Criterion) %>%
+      write.csv(file = file, row.names = FALSE, quote = FALSE)
+  }
+)
+
+
+output$data_exploration_total_score_db <- downloadHandler(
+  filename = function() "total_score.csv",
+  content = function(file) {
+    data_exploration_variables_input() %>%
+      select(total_score = "Total score") %>%
+      write.csv(file = file, row.names = FALSE, quote = FALSE)
+  }
+)
+
+
+output$data_exploration_observed_score_db <- downloadHandler(
+  filename = function() "observed_score.csv",
+  content = function(file) {
+    data_exploration_variables_input() %>%
+      select(observed_score = "Observed score vector") %>%
+      write.csv(file = file, row.names = FALSE, quote = FALSE)
+  }
+)
